﻿package com  {

	import flash.events.*;
	import flash.filesystem.File;
	import flash.net.URLRequest;
	
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.ID3Info;
	
	import com.Song;
	import com.TrackListHandler;
	
	/**
	* @author Michael Zalla
	* 
	* This class defines a object which the user interacts with via a visual interface.
	* The SongAlbum creates a list of Song objects and controls their playback based on user input.
	* It is responsible for reading audio files from the disk and assembling the Song objects. The
	* application must access the SoundChannel through SongAlbum
	*/	
	public class SongAlbum {

		public static var SONG_CHANGE:String = "SONG_CHANGE";

		private var _dispatcher:EventDispatcher;

		private var _trackList:Vector.<Song> = new Vector.<Song>(0,false); //Empty, but of unfixed length
		private var _trackListHandler:TrackListHandler;
		private var _channel:SoundChannel = null;
		private var _currentSongIndex:int = 0;
		private var _currentSongPosition:int = 0; //Only updates when we call pause()
		
		private var _hasPlayed:Boolean = false;
		private var _isPaused:Boolean = true;
		
		/**
		* Constructor
		* @param audioPath Points to the location of application's song files
		*/
		public function SongAlbum(dispatcher_:EventDispatcher, audioPath:String="audio"):void {

			_dispatcher = dispatcher_;

			/* Read song files from application root */
			var root:File = File.applicationDirectory;
			var audio:File = root.resolvePath(audioPath);
			var files:Array = audio.getDirectoryListing();
			
			/* Populate _trackList */
			for each(var file in files) {
				//We only want MP3 files
				if(file.name.indexOf(".mp3") >= 0) {
					var audioReq = new URLRequest(audioPath + "/" + file.name);
					//But we also want the lyrics in its matching text file
					var lyricsReq = new URLRequest(audioPath + "/" + file.name.substr(0,file.name.indexOf(".")) + ".txt");
					var song = new Song(audioReq,lyricsReq);
						song.addEventListener(Event.ID3, ID3Handler);
					_trackList.push(song);					
				}
			}
		
		}
		
		/**
		* Fires when a Song's ID3 info is recieved
		* Updates the Song's trackTitle and trackNumber based on the ID3 data
		*/
		private function ID3Handler(event):void {
			
			var song:Song = event.target as Song;
			
			//Update track title
			song.trackTitle = song.id3.songName;
			//Parse track number (from ID3 string)
			var n:String = song.id3.track;
			    n = n.substr(0,n.indexOf(("/" + _trackList.length) as String,0));
			//Update track number
			song.trackNumber = int(n);
			
			//Call _trackListHandler's addTrack() method, adding a new TrackItem MovieClip to the 'tracks' content
			if(_trackListHandler != null) {
				var trackData:Array = [song.trackTitle, song.trackNumber]
				_trackListHandler.addTrack(trackData);
			}
			
			//trace("Loaded ID3 info for " + song.trackTitle + " (Track " + song.trackNumber + ")");
		
			if(song.trackNumber == 1) {
				/* Populate the lyrics container on startup (this is ugly but easy ...) */
				this.play();
				this.pause();			
			}
		
		}
		
		/**
		*
		*/
		public function addTrackListHandler(handler:TrackListHandler) {
			_trackListHandler = handler;
		}
		
		/**
		* Resume (or begin) playback of the album, re-assigning 'channel' to a new SoundChannel
		* object returned when we call play() on the current Song object. Remember that a SoundChannel
		* object has no play() method, only stop()!
		*
		* By default, play() will resume playing the current song. However, a specific track can be
		* specified by passing its track title (_currentSongPosition will be reset to 0)
		*/
		public function play(trackTitle:String="default"):void {
			
			if(_isPaused) {

				var currentSongTitle:String = _trackList[_currentSongIndex].trackTitle;
				if(trackTitle != "default" && trackTitle != currentSongTitle) {
					//A specific new track was requested
					for each(var song:Song in _trackList) {
						if(song.trackTitle == trackTitle) {
							_currentSongPosition = 0;
							_currentSongIndex = _trackList.indexOf(song);
							/* Dispatch an event to let the LyricsHandler class know to update */
							_dispatcher.dispatchEvent(new Event(SongAlbum.SONG_CHANGE));
						}
					}
				}
				if(_channel != null)
					_channel.stop();
				
				//Used to update the lyrics when the play button is first clicked by the user
				if(!_hasPlayed) {
					_hasPlayed = true;
					_dispatcher.dispatchEvent(new Event(SongAlbum.SONG_CHANGE));
				}
					
				_isPaused = false;
				_channel = _trackList[_currentSongIndex].play(_currentSongPosition);
				
				/* Listener for a SoundChannel.soundComplete event, so we know when to skip to the next song */
				_channel.addEventListener(Event.SOUND_COMPLETE, function(e:Event){ next(); });
				
			}

		}
		
		/**
		* Pause playback of the album, updating currentSongPosition before stopping the SoundChannel
		*/
		public function pause():void {
			if(_channel != null) {
				_isPaused = true;
				_currentSongPosition = _channel.position;
				_channel.stop();
			}
		}
		
		/**
		* Begin playing the following song on the album. To do this correctly, we have to first
		* pause any playback, reset currentSongPosition to zero, and update currentSongIndex
		*/		
		public function next():void {
			pause();
			_currentSongPosition = 0;
			_currentSongIndex = (_currentSongIndex == (_trackList.length - 1)) ? 0 : _currentSongIndex + 1;
			/* Dispatch an event to let the LyricsHandler class know to update */
			_dispatcher.dispatchEvent(new Event(SongAlbum.SONG_CHANGE));
			/* And play ... */
			play();
		}
		
		/**
		* Begin playing the preceeding song on the album. To do this correctly, we have to first
		* pause any playback, reset currentSongPosition to zero, and update currentSongIndex
		*/
		public function previous():void {
			pause();
			_currentSongPosition = 0;
			_currentSongIndex = (_currentSongIndex == 0) ? (_trackList.length - 1) : _currentSongIndex - 1;
			/* Dispatch an event to let the LyricsHandler class know to update */
			_dispatcher.dispatchEvent(new Event(SongAlbum.SONG_CHANGE));
			/* And play ... */
			play();
		}	

		/* The following functions are mainly here to help program a song scrubber/slider */
		
		public function setPlayhead(position:Number):void {
			var isPaused = _isPaused;
			pause();
			_currentSongPosition = position;
			if(!isPaused) play();
		}
		
		public function getCurrentSongPercentComplete():Number {
			var currentSong:Song = _trackList[_currentSongIndex] as Song;
			var ret:Number = (_channel != null) ? (_channel.position / currentSong.duration) : 0;
			return ret;
		}

		public function getCurrentSongData():Object {
			
			if(_trackList[_currentSongIndex] != null) {
				var songData:Object = { trackTitle: _trackList[_currentSongIndex].trackTitle,
										trackNumber: _trackList[_currentSongIndex].trackNumber,
										duration: _trackList[_currentSongIndex].duration,
										lyrics: _trackList[_currentSongIndex].lyrics }
				return songData;
			}
			else {
				return new Object();
			}

		}
		
	}
	
}